@@ -10,17 +10,17 @@ BASE_URL = "https://m.tiktok.com/"
 
 
 class TikTokApi:
-    def __init__(self, debug=False, request_delay=None, executablePath=None):
+    def __init__(self, **kwargs):
         """The TikTokApi class. Used to interact with TikTok.
 
         :param debug: If you want debugging to be enabled.
         :param request_delay: The amount of time to wait before making a request.
         :param executablePath: The location of the chromedriver.exe
         """
-        self.debug = debug
-        if debug:
+        self.debug = kwargs.get("debug", False)
+        if self.debug:
             print("Class initialized")
-        self.executablePath = executablePath
+        self.executablePath = kwargs.get("executablePath", None)
 
         self.userAgent = (
             "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
@@ -29,7 +29,7 @@ class TikTokApi:
         )
 
         # Get Browser Params
-        b = browser("newParam", newParams=True, executablePath=self.executablePath)
+        b = browser("newParam", newParams=True, **kwargs)
 
         try:
             self.timezone_name = self.__format_new_params__(b.timezone_name)
@@ -40,7 +40,7 @@ class TikTokApi:
             self.width = b.width
             self.height = b.height
         except Exception as e:
-            if debug:
+            if self.debug:
                 print("The following error occurred, but it was ignored.")
                 print(e)
 
@@ -52,7 +52,7 @@ class TikTokApi:
             self.width = "1920"
             self.height = "1080"
 
-        self.request_delay = request_delay
+        self.request_delay = kwargs.get("request_delay", None)
 
     def getData(self, b, **kwargs) -> dict:
         """Returns a dictionary of a response from TikTok.
@@ -135,13 +135,21 @@ class TikTokApi:
         ) = self.__process_kwargs__(kwargs)
         query = {"verifyFp": b.verifyFp, "_signature": b.signature}
         url = "{}&{}".format(b.url, urlencode(query))
+        print(b.did)
         r = requests.get(
             url,
             headers={
-                "method": "GET",
-                "accept-encoding": "gzip, deflate, br",
-                "referer": b.referrer,
-                "user-agent": b.userAgent,
+                "Accept": "*/*",
+                "Accept-Encoding": "identity;q=1, *;q=0",
+                "Accept-Language": "en-US;en;q=0.9",
+                "Cache-Control": "no-cache",
+                "Connection": "keep-alive",
+                "cookie": "tt_webid_v2=" + b.did,
+                "Host": url.split("/")[2],
+                "Pragma": "no-cache",
+                "Range": "bytes=0-",
+                "Referer": "https://www.tiktok.com/",
+                "User-Agent": b.userAgent,
             },
             proxies=self.__format_proxy(proxy),
         )
@@ -185,12 +193,7 @@ class TikTokApi:
             api_url = "{}api/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(
-                api_url,
-                language=language,
-                proxy=proxy,
-                executablePath=self.executablePath,
-            )
+            b = browser(api_url, **kwargs)
             res = self.getData(b, proxy=proxy)
 
             for t in res.get("items", []):
@@ -270,7 +273,7 @@ class TikTokApi:
             api_url = "{}api/discover/{}/?{}&{}".format(
                 BASE_URL, prefix, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+            b = browser(api_url, **kwargs)
             data = self.getData(b, proxy=proxy)
 
             if "userInfoList" in data.keys():
@@ -339,7 +342,7 @@ class TikTokApi:
             api_url = "{}api/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+            b = browser(api_url, **kwargs)
             res = self.getData(b, proxy=proxy)
             if "items" in res.keys():
                 for t in res["items"]:
@@ -386,6 +389,7 @@ class TikTokApi:
             proxy=proxy,
             language=language,
             region=region,
+            **kwargs,
         )
 
     def userPage(self, userID, secUID, page_size=30, **kwargs) -> dict:
@@ -422,7 +426,7 @@ class TikTokApi:
             region,
             language,
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)
 
     def getUserPager(self, username, page_size=30, **kwargs):
@@ -522,7 +526,7 @@ class TikTokApi:
             api_url = "{}api/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+            b = browser(api_url, **kwargs)
             res = self.getData(b, proxy=proxy)
 
             try:
@@ -577,6 +581,7 @@ class TikTokApi:
             proxy=proxy,
             language=language,
             region=region,
+            **kwargs,
         )
 
     def bySound(self, id, count=30, **kwargs) -> dict:
@@ -622,7 +627,7 @@ class TikTokApi:
             api_url = "{}share/item/list?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+            b = browser(api_url, **kwargs)
             res = self.getData(b, proxy=proxy)
 
             for t in res["body"]["itemListData"]:
@@ -659,7 +664,7 @@ class TikTokApi:
         api_url = "{}api/music/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)
 
     def byHashtag(self, hashtag, count=30, **kwargs) -> dict:
@@ -686,7 +691,12 @@ class TikTokApi:
         id = self.getHashtagObject(hashtag)["challengeInfo"]["challenge"]["id"]
         response = []
 
-        while len(response) < count:
+        required_count = count
+
+        while len(response) < required_count:
+            if count > maxCount:
+                count = maxCount
+            added = 0
             query = {
                 "count": count,
                 "challengeID": id,
@@ -699,20 +709,24 @@ class TikTokApi:
             api_url = "{}api/challenge/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+            b = browser(api_url, **kwargs)
             res = self.getData(b, proxy=proxy, language=language)
 
-            for t in res["itemList"]:
-                response.append(t)
+            try:
+                for t in res["itemList"]:
+                    response.append(t)
+                    added += 1
+            except:
+                print(res)
 
             if not res["hasMore"]:
                 if self.debug:
                     print("TikTok isn't sending more TikToks beyond this point.")
                 return response
 
-            offset = len(response)
+            offset += added
 
-        return response[:count]
+        return response[:required_count]
 
     def getHashtagObject(self, hashtag, **kwargs) -> dict:
         """Returns a hashtag object.
@@ -735,7 +749,7 @@ class TikTokApi:
         api_url = "{}api/challenge/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)
 
     def getHashtagDetails(self, hashtag, **kwargs) -> dict:
@@ -759,10 +773,10 @@ class TikTokApi:
         api_url = "{}node/share/tag/{}?{}&{}".format(
             BASE_URL, quote(hashtag), self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)
 
-    def getRecommendedTikToksByVideoID(self, id, **kwargs) -> dict:
+    def getRecommendedTikToksByVideoID(self, id, count=30, **kwargs) -> dict:
         """Returns a dictionary listing reccomended TikToks for a specific TikTok video.
 
         :param id: The id of the video to get suggestions for.
@@ -779,22 +793,48 @@ class TikTokApi:
             maxCount,
             offset,
         ) = self.__process_kwargs__(kwargs)
-        query = {
-            "count": 24,
-            "id": id,
-            "type": 0,
-            "secUid": "",
-            "maxCursor": maxCursor,
-            "minCursor": minCursor,
-            "shareUid": "",
-            "recType": 3,
-            "language": language,
-        }
-        api_url = "{}share/item/list?{}&{}".format(
-            BASE_URL, self.__add_new_params__(), urlencode(query)
-        )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
-        return self.getData(b, proxy=proxy)["body"]
+
+        response = []
+        first = True
+
+        while len(response) < count:
+            if count < maxCount:
+                realCount = count
+            else:
+                realCount = maxCount
+
+            query = {
+                "count": realCount,
+                "id": 1,
+                "secUid": "",
+                "maxCursor": maxCursor,
+                "minCursor": minCursor,
+                "sourceType": 12,
+                "appId": 1233,
+                "region": region,
+                "priority_region": region,
+                "language": language,
+            }
+            api_url = "{}api/recommend/item_list/?{}&{}".format(
+                BASE_URL, self.__add_new_params__(), urlencode(query)
+            )
+            b = browser(api_url, **kwargs)
+            res = self.getData(b, proxy=proxy)
+
+            for t in res.get("items", []):
+                response.append(t)
+
+            if not res["hasMore"] and not first:
+                if self.debug:
+                    print("TikTok isn't sending more TikToks beyond this point.")
+                return response[:count]
+
+            realCount = count - len(response)
+            maxCursor = res["maxCursor"]
+
+            first = False
+
+        return response[:count]
 
     def getTikTokById(self, id, **kwargs) -> dict:
         """Returns a dictionary of a specific TikTok.
@@ -813,6 +853,7 @@ class TikTokApi:
             maxCount,
             offset,
         ) = self.__process_kwargs__(kwargs)
+        did = kwargs.get("custom_did", None)
         query = {
             "itemId": id,
             "language": language,
@@ -820,7 +861,7 @@ class TikTokApi:
         api_url = "{}api/item/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)
 
     def getTikTokByUrl(self, url, **kwargs) -> dict:
@@ -840,6 +881,7 @@ class TikTokApi:
             maxCount,
             offset,
         ) = self.__process_kwargs__(kwargs)
+        custom_did = kwargs.get("custom_did", None)
         if "@" in url and "/video/" in url:
             post_id = url.split("/video/")[1].split("?")[0]
         else:
@@ -848,7 +890,10 @@ class TikTokApi:
                 "https://www.tiktok.com/@therock/video/6829267836783971589"
             )
 
-        return self.getTikTokById(post_id, language=language, proxy=proxy)
+        return self.getTikTokById(
+            post_id,
+            **kwargs,
+        )
 
     def discoverHashtags(self, **kwargs) -> dict:
         """Discover page, consists challenges (hashtags)
@@ -868,7 +913,7 @@ class TikTokApi:
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)["body"][1]["exploreList"]
 
     def discoverMusic(self, **kwargs) -> dict:
@@ -889,7 +934,7 @@ class TikTokApi:
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)["body"][2]["exploreList"]
 
     def getUserObject(self, username, **kwargs) -> dict:
@@ -932,7 +977,7 @@ class TikTokApi:
         api_url = "{}api/user/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
         return self.getData(b, proxy=proxy)["userInfo"]
 
     def getSuggestedUsersbyID(
@@ -963,7 +1008,7 @@ class TikTokApi:
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
 
         res = []
         for x in self.getData(b, proxy=proxy)["body"][0]["exploreList"]:
@@ -1033,7 +1078,7 @@ class TikTokApi:
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
 
         res = []
         for x in self.getData(b, proxy=proxy)["body"][1]["exploreList"]:
@@ -1104,7 +1149,7 @@ class TikTokApi:
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        b = browser(api_url, proxy=proxy, executablePath=self.executablePath)
+        b = browser(api_url, **kwargs)
 
         res = []
         for x in self.getData(b, proxy=proxy)["body"][2]["exploreList"]:
@@ -1168,8 +1213,8 @@ class TikTokApi:
             try:
                 api_url = data["itemInfos"]["video"]["urls"][0]
             except Exception:
-                api_url = data["itemInfo"]["itemStruct"]["video"]["downloadAddr"]
-        return self.get_Video_By_DownloadURL(api_url, proxy=proxy)
+                api_url = data["itemInfo"]["itemStruct"]["video"]["playAddr"]
+        return self.get_Video_By_DownloadURL(api_url, **kwargs)
 
     def get_Video_By_DownloadURL(self, download_url, **kwargs) -> bytes:
         """Downloads video from TikTok using download url in a TikTok object
@@ -1186,20 +1231,25 @@ class TikTokApi:
             maxCount,
             offset,
         ) = self.__process_kwargs__(kwargs)
-        b = browser(download_url, proxy=proxy, executablePath=self.executablePath)
-        return self.getBytes(b, proxy=proxy)
+        b = browser(download_url, **kwargs)
+        return self.getBytes(b, **kwargs)
 
-    def get_Video_By_Url(
-        self, video_url, return_bytes=0, chromedriver_path=None
-    ) -> bytes:
-        """(DEPRECRATED)
-        Gets the source url of a given url for a tiktok
+    def get_Video_By_Url(self, video_url, **kwargs) -> bytes:
+        (
+            region,
+            language,
+            proxy,
+            minCursor,
+            maxCursor,
+            maxCount,
+            offset,
+        ) = self.__process_kwargs__(kwargs)
 
-        video_url - the url of the video
-        return_bytes - 0 is just the url, 1 is the actual video bytes
-        chromedriver_path - path to your chrome driver executable
-        """
-        raise Exception("Deprecated. Other Methods Work Better.")
+        tiktok_schema = self.getTikTokByUrl(video_url, **kwargs)
+        download_url = tiktok_schema["itemInfo"]["itemStruct"]["video"]["downloadAddr"]
+
+        b = browser(download_url, **kwargs)
+        return self.getBytes(b, **kwargs)
 
     def get_Video_No_Watermark(self, video_url, return_bytes=0, **kwargs) -> bytes:
         """Gets the video with no watermark
@@ -1264,12 +1314,7 @@ class TikTokApi:
                 "&media_type=4&vr_type=0"
             ).format(key)
 
-            b = browser(
-                cleanVideo,
-                find_redirect=True,
-                proxy=proxy,
-                executablePath=self.executablePath,
-            )
+            b = browser(cleanVideo, **kwargs)
             print(b.redirect_url)
             if return_bytes == 0:
                 return b.redirect_url
@@ -1332,7 +1377,7 @@ class TikTokApi:
         proxy = kwargs.get("proxy", None)
         maxCursor = kwargs.get("before", 0)
         minCursor = kwargs.get("after", 0)
-        maxCount = kwargs.get("maxCount", 50)
+        maxCount = kwargs.get("maxCount", 35)
         offset = kwargs.get("offset", 0)
 
         return region, language, proxy, minCursor, maxCursor, maxCount, offset
